/*********************************************************************
 *                
 * Filename:      irattach.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Sun Dec  7 23:21:05 1997
 * Modified at:   Mon Apr  8 11:30:28 2002
 * Modified by:   Ronny Arild <ronny.arild@thalesgroup.no>
 * Sources:       
 *
 *     Copyright (c) 1997, 1999-2000 Dag Brattli <dagb@cs.uit.no>, 
 *     All Rights Reserved.
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *
 *     Neither Dag Brattli nor University of Tromsadmit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *
 ********************************************************************/

/**********************************************************************
* Modified by
* (C) NEC CASIO Mobile Communications, Ltd. 2011
**********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <syslog.h>
#include <termios.h>
#include <unistd.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/utsname.h>

#include <irda.h>

#ifndef N_IRDA
#define N_IRDA 11 
#endif 

#ifndef AF_IRDA
#define AF_IRDA 23
#endif


extern void fork_now(int ttyfd);
extern int set_sysctl_param(char *name, char *value);
extern int execute(char *msg, char *cmd);


#define VERSION "0.9.17 (19.02.2006) Dag Brattli/Jean Tourrilhes"

extern char *optarg;
extern int optind;

static int devfd = -1;		
static int fdflags = -1;	
static int initfdflags = -1;	
static int initdisc = -1;	
static struct termios termsave;	
static int termvalid = -1;	


char *pidfile = "/var/run/irattach.pid";


struct irtty_info {
	char name[6];
};

#define IRTTY_IOC_MAGIC 'e'
#define IRTTY_IOCTDONGLE  _IO(IRTTY_IOC_MAGIC, 1)
#define IRTTY_IOCGNAME   _IOR(IRTTY_IOC_MAGIC, 2, struct irtty_info)
#define IRTTY_IOC_MAXNR  2  

struct irtty_info info;

char device[20];

struct dongle_list_s {
	int   id;
	char *dongle;
};
struct dongle_list_s dongle_list[] = {
	{ IRDA_ESI_DONGLE,		"esi" },
	{ IRDA_TEKRAM_DONGLE,		"tekram" },
	{ IRDA_ACTISYS_DONGLE,		"actisys" },
	{ IRDA_ACTISYS_PLUS_DONGLE,	"actisys+" },
	{ IRDA_GIRBIL_DONGLE,		"girbil" },
	{ IRDA_LITELINK_DONGLE,		"litelink" },
	{ IRDA_AIRPORT_DONGLE,		"airport" },
	{ IRDA_OLD_BELKIN_DONGLE,	"old_belkin" },
	{ IRDA_EP7211_IR,		"ep7211" },
	{ IRDA_MCP2120_DONGLE,		"mcp2120" },
	{ IRDA_ACT200L_DONGLE,		"act200l" },
	{ IRDA_MA600_DONGLE,		"ma600" },
	{ IRDA_TOIM3232_DONGLE,		"toim3232" },
	{ IRDA_MSMSIR_DONGLE,		"msmsir" },
	{ -1,				NULL }
};


#define PROC_NET_DEV	"/proc/net/dev"









static int modify_flags(char *dev, int set, int clear)
{
		struct ifreq ifr;
		int sockfd;

		
        sockfd = socket(AF_IRDA, SOCK_STREAM, 0);
        if (sockfd < 0) {
                syslog (LOG_WARNING, "socket(AF_IRDA): %m");
                printf("socket(AF_IRDA): %m");
				return(-1);
				
        }

		
        strncpy(ifr.ifr_name, dev, IFNAMSIZ);
		if (ioctl(sockfd, SIOCGIFFLAGS, &ifr) < 0) {
				syslog (LOG_WARNING, "ioctl(SIOCGIFFLAGS): %m");
				printf("ioctl(SIOCGIFFLAGS): %m");
				close (sockfd);
				return -1;
		}

		
        ifr.ifr_flags |= set;
		ifr.ifr_flags &= ~clear;

		
		strncpy(ifr.ifr_name, dev, IFNAMSIZ);
        if (ioctl(sockfd, SIOCSIFFLAGS, &ifr) < 0) {
				syslog (LOG_WARNING, "ioctl(SIOCSIFFLAGS): %m");
				printf("ioctl(SIOCSIFFLAGS): %m");
				close (sockfd);
                return -1;
        }
		close(sockfd);
        return 0;	
}







static inline int ifup(char *dev)
{
	syslog(LOG_INFO, "Starting device %s", dev);
	printf("Starting device %s\n", dev);
	return(modify_flags(dev, IFF_UP|IFF_RUNNING, 0));
}







static inline int ifdown(char *dev)
{
	syslog(LOG_INFO, "Stopping device %s", dev);
	printf("Stopping device %s\n", dev);
	return(modify_flags(dev, 0, IFF_UP));
}






void cleanup(int signo)
{
	printf("cleanup %d\n",signo);

	switch (signo) {
	case SIGTERM:
	case SIGINT:
	        syslog(LOG_INFO, "got SIGTERM or SIGINT\n");
	        printf("got SIGTERM or SIGINT\n");
		break;
	case SIGHUP:
	        syslog(LOG_INFO, "got SIGHUP\n");
	        printf("got SIGHUP\n");
		break;
	default:
		break;
	}
	ifdown(device);

	
	if (devfd != -1) {
		fdflags &= ~O_NONBLOCK;
		if (fcntl (devfd, F_SETFL, fdflags) != 0) {
			syslog (LOG_ERR, "fcntl: fdflags: %m");
			printf("fcntl: fdflags: %m");
		}
		if (initdisc != -1)
			if (ioctl(devfd, TIOCSETD, &initdisc) < 0){
				fprintf(stderr, "Oups ! Can't set back original link discipline...\n");
				syslog(LOG_ERR, "ioctl: set_inidisc: %m");
				printf("ioctl: set_inidisc: %m");
			}
		if (termvalid != -1)
			if (tcsetattr(devfd, TCSANOW, &termsave) != 0) {
				syslog (LOG_ERR, "tcsetattr: %m");
				printf("tcsetattr: %m");
			}
		if (fcntl (devfd, F_SETFL, initfdflags) != 0) {
			syslog (LOG_ERR, "fcntl: initfdflags: %m");
			printf("fcntl: initfdflags: %m");
		}
		close(devfd);
		devfd = -1;
	}

	
	unlink(pidfile);

	syslog(LOG_INFO, "exiting ...\n");
	printf("exiting ...\n");

	if (signo == -1)
		exit(1);
	else
		exit(0);
}



static inline void clean_exit(int status)
{
	printf("cleanup exit\n");

	cleanup(status);
	
}

static void print_usage(void)
{
	int i;
	fprintf(stderr, "Usage: irattach <dev> [-d dongle] [-s] [-b] [-v] [-h]\n");
	fprintf(stderr, "       <dev> is tty name, device name or module name\n");
	fprintf(stderr, "Dongles supported :\n");
	for (i = 0; dongle_list[i].dongle != NULL; i++)
		fprintf(stderr, "\t%s\n", dongle_list[i].dongle);
	fprintf(stderr, "\n");
}









static int load_module(char *name)
{
	char msg[128], cmd[512];
	int ret;
	
	sprintf(msg, "Trying to load module %s", name);
	sprintf(cmd, "/sbin/modprobe %s", name);

	ret = execute(msg, cmd);

	return ret;
}





static char *
get_irdevname(char *	name,	
	      int	nsize,	
	      char *	buf)	
{
	char *	end;

	
	while(isspace(*buf))
		buf++;

	
	if(strncmp(buf, "irda", 4))
		return(NULL);
	
	end = strchr(buf, ':');

	
	if((end == NULL) || (((end - buf) + 1) > nsize))
		return(NULL);

	
	memcpy(name, buf, (end - buf));
	name[end - buf] = '\0';

	return(end + 2);
}







static int get_devlist(char ifnames[][IFNAMSIZ + 1],
		       int maxchars, int maxnames)
{
	char		buff[1024];
	FILE *		fh;
	int		ifnum;
	
	
	fh = fopen(PROC_NET_DEV, "r");

	if(fh == NULL)
		return(-1);

	
	ifnum = 0;

	
	fgets(buff, sizeof(buff), fh);
	fgets(buff, sizeof(buff), fh);

	
	while(fgets(buff, sizeof(buff), fh)) {
		
		if(get_irdevname(ifnames[ifnum], maxchars, buff)) {
			ifnum++;
		}
		if(ifnum > maxnames) {
			fclose(fh);
			return(-1);
		}
	}
	fclose(fh);
	return(ifnum);
}







static inline int get_module_devices(char *	modname)
{
	char	before_names[10][IFNAMSIZ + 1];
	int	before_num;
	char	after_names[15][IFNAMSIZ + 1];
	int	after_num;
	int	firstone = -1;
	int	pid;
	int	i, j;
	int	ret;

	
	before_num = get_devlist(before_names, IFNAMSIZ + 1, 10);
	if(before_num < 0) {
		fprintf(stderr, "Could not get device name list.\n");
		exit(-1);
	}

	
	ret = load_module(modname);
	if(ret) {
		fprintf(stderr, "Invalid module name [%s] !\n", device);
		print_usage();
		exit(-1);
	}

	
	after_num = get_devlist(after_names, IFNAMSIZ + 1, 15);
	if(after_num <= 0) {
		fprintf(stderr, "Could not get device name list.\n");
		exit(-1);
	}

	
	for(i = 0; i < after_num; i++) {
		
		for(j = 0; j < before_num; j++) {
			if(!strcmp(before_names[j], after_names[i]))
				break;
		}
		
		if(j == before_num) {
			
			if(firstone >= 0) {
				fprintf(stderr,
					"Found additional interface [%s]\n",
					after_names[i]);
				

				pid = fork();
				
				if(!pid) {
					
					strcpy(device, after_names[i]);
					
					return(0);
				}
				


			} else {
				
				firstone = i;
				strcpy(device, after_names[i]);
				fprintf(stderr, "Found interface [%s]\n",
					device);
			}
		}
	}
	return(0);
}








static void establish_irda (int ttyfd) 
{
	int irdadisc = N_IRDA;
	
	if (ioctl(ttyfd, TIOCEXCL, 0) < 0) {
		syslog (LOG_WARNING, "ioctl(TIOCEXCL): %m");
		printf("ioctl(TIOCEXCL): %m");
	}
	
	if (ioctl(ttyfd, TIOCGETD, &initdisc) < 0) {
		syslog(LOG_ERR, "ioctl(TIOCGETD): %m");
		printf("ioctl(TIOCGETD): %m");
		clean_exit(-1);
	}
	
	if (ioctl(ttyfd, TIOCSETD, &irdadisc) < 0){
		fprintf(stderr,  
			 "Maybe you don't have IrDA support in your kernel?\n");
		syslog(LOG_ERR, "irattach: tty: set_disc(%d): %s\n", 
			irdadisc, strerror(errno));
		clean_exit(-1);
	}
}







static int tty_configure(struct termios *tios) 
{
	tios->c_cflag     = CS8|CREAD|B9600|CLOCAL;
	
	
 	tios->c_iflag     = IGNBRK | IGNPAR;
	tios->c_oflag     = 0;
	tios->c_lflag     = 0; 
	tios->c_cc[VMIN]  = 1; 
	tios->c_cc[VTIME] = 0; 
	
	return(0);
}







static void init_irda_ldisc(int ttyfd) 
{
	struct termios tios;
	
	
	if (tcgetattr(ttyfd, &tios) != 0) {
		syslog (LOG_ERR, "tcgetattr: %m");
		printf("tcgetattr: %m");
		clean_exit(-1);
	}
	
	memcpy(&termsave, &tios, sizeof(struct termios));
	termvalid = 0;

	tty_configure(&tios);
	
	
	if (tcsetattr(ttyfd, TCSAFLUSH, &tios) < 0) {
		syslog(LOG_ERR, "tcsetattr: %m");
		printf("tcsetattr: %m");
		clean_exit(-1);
	}
}








static void start_tty(int ttyfd) 
{
	int cloexec;

	printf("start tty\n");

	
	fdflags &= ~O_NONBLOCK;
	if (fcntl(ttyfd, F_SETFL, fdflags) == -1) {
		syslog(LOG_WARNING, "Couldn't set device fd flags: %m");
		printf("Couldn't set device fd flags: %m");
	}
		
	








	if ((cloexec = fcntl(devfd, F_GETFD)) == -1) {
		syslog(LOG_ERR,
		       "Couldn't get device fd close-on-exec flag: %m");
		printf("Couldn't get device fd close-on-exec flag: %m");
		clean_exit(-1);
	}
	cloexec |= FD_CLOEXEC;
	if (fcntl(devfd, F_SETFD, cloexec) == -1) {
		syslog(LOG_ERR,
		       "Couldn't set device fd close-on-exec flag: %m");
		printf("Couldn't set device fd close-on-exec flag: %m");
		clean_exit(-1);
	}

		
	init_irda_ldisc(ttyfd);
	establish_irda(ttyfd);

	
	sleep(1);
	
	


	if (fcntl(ttyfd, F_SETFL, fdflags | O_NONBLOCK) == -1) {
		syslog(LOG_ERR, 
		       "Couldn't set device to non-blocking mode: %m");
		printf("Couldn't set device to non-blocking mode: %m");
		clean_exit(-1);
	}
}








static int open_tty(char *dev) 
{
	int ttyfd;

	printf("open tty\n");

	
	ttyfd = open(dev, O_NONBLOCK | O_RDWR, 0);
	if (ttyfd < 0) {
		fprintf(stderr, "Failed to open device %s: %s\n",
			dev, strerror(errno));
		exit(-1);
	}

	


	if ((fdflags = fcntl(ttyfd, F_GETFL)) == -1) {
		fprintf(stderr, "Couldn't get device %s flags: %s\n",
			dev, strerror(errno));
		close(ttyfd);
		exit(-1);
	}
	initfdflags = fdflags;

	return(ttyfd);
}









static inline void attach_dongle(int ttyfd, char *dev, int dongle)
{

	printf("attach dongle\n");

	
	if(ttyfd != -1) {
		
		ioctl(ttyfd, IRTTY_IOCTDONGLE, dongle);
		printf("Attachcmd=0x%X dongle=0x%X\n", IRTTY_IOCTDONGLE,dongle);
	} else {
		
		int	sockfd;
		struct ifreq ifr;

		
		sockfd = socket(AF_IRDA, SOCK_STREAM, 0);
		if (sockfd < 0) {
			perror("socket");
			clean_exit(-1);
		}

		
		ifr.ifr_data = (void *) dongle;
		strncpy(ifr.ifr_name, dev, IFNAMSIZ);
		if (ioctl(sockfd, SIOCSDONGLE, &ifr) < 0) {
			perror("ioctl");
			close(sockfd);
			clean_exit(-1);
		}

		
		close(sockfd);
	}
}







static inline int get_dongle(char *dongle)
{
	int i;
	for (i = 0; dongle_list[i].dongle != NULL; i++) {
		if (strcmp(dongle_list[i].dongle, dongle) == 0)
		       	return dongle_list[i].id;
	}
	return -1;
}









int main(int argc, char *argv[]) 
{
	struct utsname buf;
	int	tty = 0;	
	int	modname = 0;	
	int	dongle = -1;	

	int	discovery = 1;	
	int	c;
	int	ret;
	int     daemonize = 1;  

	
	if ((argc < 2) || (argc > 5)) {
		print_usage();
		exit(-1);
	}

	

	strncpy(device, argv[1], 20);
	device[20] = '\0';

	
	

	while ((c = getopt(argc, argv, "d:hsvb")) != -1) {
		printf("option=%x\n",c);
		switch (c) {
		case 's':
		       	
			discovery = 1;
			break;
		case 'd':
			dongle = get_dongle(optarg);
			if (dongle == -1) {
				fprintf(stderr,
				       "Sorry, dongle not supported yet!\n");
				print_usage();
				exit(-1);
			}	
			break;
		case 'v':
			printf("Version: %s\n", VERSION);
			exit(0);
		case 'h':
			print_usage();
			exit(0);
		case 'b':
		       	
			daemonize = 0;
			break;
		default:
			print_usage();
			exit(-1);
		}
	}

	dongle = get_dongle("msmsir");
	printf("dongle=%d\n",dongle);

	
	if(strncmp("/dev", device, 4) == 0) {
		
		tty = 1;

		
		devfd = open_tty(device);
	} else {
		
		if((strncmp("irda", device, 4) == 0) &&
		   (isdigit(device[4]))) {
			


			load_module(device);
		} else {
			
			modname = 1;

			
			
			get_module_devices(device);
		}
	}












	
	



	

	printf("start signal\n");










	printf("exit signal\n");

	
	
	if (tty) {
		
		start_tty(devfd);

		
		if (ioctl(devfd, IRTTY_IOCGNAME, &info) < 0) {
			syslog(LOG_ERR, "Are you using an old kernel?");
			clean_exit(-1);
		}
		strncpy(device, info.name, 20);
	}

	
	




	

	printf("dongle=%d\n",dongle);

	if (dongle != -1) {
		attach_dongle(devfd, device, dongle);
  	}

	
	if (uname(&buf) == 0)
		set_sysctl_param("devname", strtok(buf.nodename, "."));

	
	if(discovery > 0) {
		printf("start discovery\n");
		set_sysctl_param("discovery", "1");
	}

	
	printf("start network interface\n");
	ret = ifup(device);
	if(ret < 0){
		printf("fail: %d\n",ret);
		clean_exit(-1);
	}
	









	printf("sleep\n");
	sleep(10);
	printf("sleep exit\n");

	return 0;
}
